;*********************************************************************
; FAT CONTROLLER ~ MIDI to CV Converter                              *
;*********************************************************************
; (c) 2002 SM-Sound / Seb Francis                                    *
;*********************************************************************
;
; Memory map, and constant declarations
;
;*********************************************************************
;                                                               
;    Connections:
;
;    RA0   <- Noise
;    RA1   -> MUX0
;    RA2   -> MUX1
;    RA3   -> MUX2
;    RA4   Open drain output!
;    RA5   -> MUX3
;
;    RB0   -> Display RS
;    RB1   -> Display R/!W
;    RB2   -> Display E
;    RB3
;    RB4   <- !PGUP
;    RB5   <- !SHIFT 
;    RB6   <- !INCR
;    RB7   <- !DECR
;
;    RC0   Used by Timer1
;    RC1   Used by Timer1
;    RC2   -> DAC !CS
;    RC3   -> DAC SCLK
;    RC4   -> MUXE
;    RC5   -> DAC DIN
;    RC6
;    RC7   <- MIDI IN
;
;    RD0-7 -> Display D0-7
;
;    RE0   -> !Gate 1
;    RE1   -> !Gate 2
;    RE2   -> !Gate 3
;                                                                  
;                                                                     
;*********************************************************************


;==================================================
; MISC

#DEFINE			_C	STATUS,C
#DEFINE			_Z	STATUS,Z
LSB			EQU	0x0
MSB			EQU	0x7


;==================================================
; MUX

#DEFINE			MUX0	PORTA,1
#DEFINE			MUX1	PORTA,2
#DEFINE			MUX2	PORTA,3
#DEFINE			MUX3	PORTA,5
#DEFINE			MUXE	PORTC,4

;==================================================
; Display

LCDRS			EQU	0x0
LCDRW			EQU	0x1
LCDE			EQU	0x2

;==================================================
; Buttons

PGUP			EQU	0x4
SHIFT			EQU	0x5
INCR			EQU	0x6
DECR			EQU	0x7

;==================================================
; DAC

DACCS			EQU	0x2


;==================================================
; Gates

GATE1			EQU	0x0
GATE2			EQU	0x1
GATE3			EQU	0x2


;==================================================
; Memory - bank 0

____SPARE____		EQU	0x0020
MENU_CONFIG_POS		EQU	0x0021		; current position in config menu
MENU_ASSIGN_POS		EQU	0x0022		; current position in assignment menu
MENU_CURRENT_CV		EQU	0x0023		; currently selected CV in assignment menu
MENU_FLAGS		EQU	0x0024		; various menu (+env) flags
  WHICH_MENU		EQU	0x0		; clear = config, set = assigment
  KEY_HELD		EQU	0x1		; set = key held for >10ms
  BIG_STEP		EQU	0x2		; set = incr/decr 16bit values in steps of 100
  SAVE_CONFIG		EQU	0x3		; set = 'save config' mode active (confirm prompt)
  SAVE_CONFIG_GO	EQU	0x4		; set = 'save config' mode active (save confirmed)
  ENV1_CALC_PENDING	EQU	0x5		; set = env1 calculation pending, so do calc in main code, and don't do anything within interupt code
  ENV2_CALC_PENDING	EQU	0x6		; set = env2 calculation pending, so do calc in main code, and don't do anything within interupt code
TMR1_COUNT		EQU	0x0025		; counter to post-scale TMR1
TMR1_COUNT_SAVE		EQU	0x0026		; saved count rate
TMR1_NUM_COUNTS		EQU	0x0027		; number of iterations at current count rate
TMR1_NUM_COUNTS_SAVE	EQU	0x0028		; saved number of iterations at current count rate
TMR1_LENGTH		EQU	0x0029		; TMR1 length pointer

LCDTEMP1		EQU	0x002A		; temporary register for LCD code
LCDTEMP2		EQU	0x002B		; temporary register for LCD code
LCDTEMP3		EQU	0x002C		; temporary register for LCD code
LCDTEMP4		EQU	0x002D		; temporary register for LCD code
LCDTEMP5		EQU	0x002E		; temporary register for LCD code
LCD_VALUE_RD		EQU	0x002F		; temporary register for LCD code
  BUSY			EQU	0x7		; lcd busy flag


; MIDI stuff
MIDI_FLAGS		EQU	0x0030
  WHICH_BYTE		EQU	0x0		; 0 - next data byte is 1st, 1 - next data byte is 2nd
  MSG_LEN		EQU	0x1		; message length: 0=2byte 1=3byte

MIDI_MSG_PROCESS_PTR	EQU	0x0031		; pointer to next MIDI message to process
MIDI_MSG_INCOMING_PTR	EQU	0x0032		; pointer to where to write next incoming MIDI message

MIDI_MSG1		EQU	0x0033		; temp buffer for message byte 1
MIDI_MSG2		EQU	0x0034		; temp buffer for message byte 2

EEPROM_FSR_SAVE		EQU	0x0035		; place to save EEPROM writing
EEPROM_MTEMP1_SAVE	EQU	0x0036		; variables while calling MIDI
EEPROM_MTEMP2_SAVE	EQU	0x0037		; processing code

MIDI_MSG_BUFFER		EQU	0x0040		; start of MIDI message ring buffer
;				0x0041
;				0x0042
;				0x0043
;				0x0044
;				0x0045
;				0x0046
;				0x0047
;				0x0048
;				0x0049
;				0x004A
;				0x004B
;				0x004C
;				0x004D
;				0x004E
MIDI_MSG_BUFFER_END	EQU	0x004F		; end of MIDI message ring buffer


; Max			EQU	0x005F


; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0  - used by interupts, so don't call maths routines in bank 0 from main loop!
; 0x00E0-0x00EF in bank 1
; 0x0160-0x016F in bank 2  - memory used for settings, don't call maths routines in bank 2!
; 0x01E0-0x01EF in bank 3
AARGB2			EQU	0x0060		; argument A, LSB
AARGB1			EQU	0x0061		; argument A
AARGB0			EQU	0x0062		; argument A, MSB
BARGB1			EQU	0x0063		; argument B, LSB
BARGB0			EQU	0x0064		; argument B, MSB
REMB1			EQU	0x0065		; remainder LSB
REMB0			EQU	0x0066		; remainder MSB
TEMP			EQU	0x0067
TEMPB3			EQU	0x0068
TEMPB1			EQU	0x0069
TEMPB0			EQU	0x006A
SIGN			EQU	0x006B
LOOPCOUNT		EQU	0x006C
FPFLAGS			EQU	0x006D
  NAN			EQU	0x0
SPARE1			EQU	0x006E
SPARE0			EQU	0x006F


; Memory accessible in all banks

W_TEMP        		EQU     0x0070        	; variable used for context saving
STATUS_TEMP		EQU	0x0071		; variable used for context saving
PCLATH_TEMP		EQU	0x0072		; variable used for context saving
FSR_TEMP		EQU	0x0073		; variable used for context saving
COUNTER			EQU	0x0074		; counter for loops
TEMP1			EQU	0x0075		; general purpose temp variable
TEMP2			EQU	0x0076		; general purpose temp variable
TEMP3			EQU	0x0077		; general purpose temp variable
TEMP4			EQU	0x0078		; general purpose temp variable
TEMP5			EQU	0x0079		; general purpose temp variable

; Temp vars for main loop
MTEMP1			EQU	0x007A
MTEMP2			EQU	0x007B
MTEMP3			EQU	0x007C
MTEMP4			EQU	0x007D
MTEMP5			EQU	0x007E
MFLAGS			EQU	0x007F
  NOTE_ON		EQU	0x0		; used by portamento code call from NoteToCV: set if note on message, clear if note off message

; Max			EQU	0x007F


;==================================================
; Memory - bank 1

NEXT_CV			EQU	0x00A0		; 1-15

CV1_VALUE_L		EQU	0x00A2
CV1_VALUE_H		EQU	0x00A3
CV2_VALUE_L		EQU	0x00A4
CV2_VALUE_H		EQU	0x00A5
CV3_VALUE_L		EQU	0x00A6
CV3_VALUE_H		EQU	0x00A7
CV4_VALUE_L		EQU	0x00A8
CV4_VALUE_H		EQU	0x00A9
CV5_VALUE_L		EQU	0x00AA
CV5_VALUE_H		EQU	0x00AB
CV6_VALUE_L		EQU	0x00AC
CV6_VALUE_H		EQU	0x00AD
CV7_VALUE_L		EQU	0x00AE
CV7_VALUE_H		EQU	0x00AF
CV8_VALUE_L		EQU	0x00B0
CV8_VALUE_H		EQU	0x00B1
CV9_VALUE_L		EQU	0x00B2
CV9_VALUE_H		EQU	0x00B3
CV10_VALUE_L		EQU	0x00B4
CV10_VALUE_H		EQU	0x00B5
CV11_VALUE_L		EQU	0x00B6
CV11_VALUE_H		EQU	0x00B7
CV12_VALUE_L		EQU	0x00B8
CV12_VALUE_H		EQU	0x00B9
CV13_VALUE_L		EQU	0x00BA
CV13_VALUE_H		EQU	0x00BB
CV14_VALUE_L		EQU	0x00BC
CV14_VALUE_H		EQU	0x00BD
CV15_VALUE_L		EQU	0x00BE
CV15_VALUE_H		EQU	0x00BF

; Random number vars
RND_SHIFT1		EQU	0x00C0
RND_SHIFT2		EQU	0x00C1
RND_SHIFT3		EQU	0x00C2
RND_SHIFT4		EQU	0x00C3
RND_TEMP		EQU	0x00C4
RND_BUFFER		EQU	0x00C5

; Note / Pitch bend stuff
NOTE1_AMT_LL		EQU	0x00C6
NOTE1_AMT_L		EQU	0x00C7
NOTE1_AMT_H		EQU	0x00C8
NOTE2_AMT_LL		EQU	0x00C9
NOTE2_AMT_L		EQU	0x00CA
NOTE2_AMT_H		EQU	0x00CB
NOTE3_AMT_LL		EQU	0x00CC
NOTE3_AMT_L		EQU	0x00CD
NOTE3_AMT_H		EQU	0x00CE

BEND1_AMT_L		EQU	0x00CF
BEND1_AMT_H		EQU	0x00D0
BEND2_AMT_L		EQU	0x00D1
BEND2_AMT_H		EQU	0x00D2
BEND3_AMT_L		EQU	0x00D3
BEND3_AMT_H		EQU	0x00D4

; Note + velocity stack pointer
NOTE_STACK_PTR		EQU	0x00D5

; NoteToCV temp registers (also used by PitchBendToDacOffset)
TMP_NOTE_VALUE		EQU	0x00D6
TMP_NOTE_VALUE_H	EQU	0x00D7

; Note off gate
NOTE_OFF_GATE_STEPS_L	EQU	0x00D8
NOTE_OFF_GATE_STEPS_H	EQU	0x00D9


; Max			EQU	0x00DF

; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0
; 0x00E0-0x00EF in bank 1
; 0x01E0-0x01EF in bank 3


;==================================================
; Memory - bank 2

; Displayed parameter values
; EEPROM address is low byte of these addresses

; Config menu
ConfigMenuValuesBase	EQU	0x0110

vBASE_CHA		EQU	0x0110

vCTRL_CHA		EQU	0x0111

vMODE			EQU	0x0112

vL_RETRIG		EQU	0x0113

vN_O_GATE		EQU	0x0114
vN_O_GATE_H		EQU	0x0115

vVELOCITY		EQU	0x0116

vPORTA			EQU	0x0117

vPORTA_T1		EQU	0x0118
vPORTA_T1_H		EQU	0x0119

vPORTA_T2		EQU	0x011A
vPORTA_T2_H		EQU	0x011B

vPORTA_T3		EQU	0x011C
vPORTA_T3_H		EQU	0x011D

vP_BEND_1		EQU	0x011E

vP_BEND_2		EQU	0x011F

vP_BEND_3		EQU	0x0120

vOCTAVE_1		EQU	0x0121

vOCTAVE_2		EQU	0x0122

vOCTAVE_3		EQU	0x0123

vLFO1_TIM		EQU	0x0124
vLFO1_TIM_H		EQU	0x0125

vLFO1_WAV		EQU	0x0126

vLFO1_SYN		EQU	0x0127

vLFO1_AMT		EQU	0x0128

vLFO1_DST		EQU	0x0129

vLFO2_TIM		EQU	0x012A
vLFO2_TIM_H		EQU	0x012B

vLFO2_WAV		EQU	0x012C

vLFO2_SYN		EQU	0x012D

vLFO2_AMT		EQU	0x012E

vLFO2_DST		EQU	0x012F

vENV1_TRG		EQU	0x0130

vENV1_ATT		EQU	0x0131
vENV1_ATT_H		EQU	0x0132

vENV1_ATL		EQU	0x0133

vENV1_DEC		EQU	0x0134
vENV1_DEC_H		EQU	0x0135

vENV1_SUS		EQU	0x0136

vENV1_REL		EQU	0x0137
vENV1_REL_H		EQU	0x0138

vENV1_DST		EQU	0x0139

vENV2_TRG		EQU	0x013A

vENV2_ATT		EQU	0x013B
vENV2_ATT_H		EQU	0x013C

vENV2_ATL		EQU	0x013D

vENV2_DEC		EQU	0x013E
vENV2_DEC_H		EQU	0x013F

vENV2_SUS		EQU	0x0140

vENV2_REL		EQU	0x0141
vENV2_REL_H		EQU	0x0142

vENV2_DST		EQU	0x0143

; Assign menu
AssignMenuValuesBase	EQU	0x0144

vSOURCE_CV1		EQU	0x0144
vSOURCE_CV2		EQU	0x0145
vSOURCE_CV3		EQU	0x0146
vSOURCE_CV4		EQU	0x0147
vSOURCE_CV5		EQU	0x0148
vSOURCE_CV6		EQU	0x0149
vSOURCE_CV7		EQU	0x014A
vSOURCE_CV8		EQU	0x014B
vSOURCE_CV9		EQU	0x014C
vSOURCE_CV10		EQU	0x014D
vSOURCE_CV11		EQU	0x014E
vSOURCE_CV12		EQU	0x014F
vSOURCE_CV13		EQU	0x0150
vSOURCE_CV14		EQU	0x0151
vSOURCE_CV15		EQU	0x0152

vAMOUNT_CV1		EQU	0x0153
vAMOUNT_CV2		EQU	0x0154
vAMOUNT_CV3		EQU	0x0155
vAMOUNT_CV4		EQU	0x0156
vAMOUNT_CV5		EQU	0x0157
vAMOUNT_CV6		EQU	0x0158
vAMOUNT_CV7		EQU	0x0159
vAMOUNT_CV8		EQU	0x015A
vAMOUNT_CV9		EQU	0x015B
vAMOUNT_CV10		EQU	0x015C
vAMOUNT_CV11		EQU	0x015D
vAMOUNT_CV12		EQU	0x015E
vAMOUNT_CV13		EQU	0x015F
vAMOUNT_CV14		EQU	0x0160
vAMOUNT_CV15		EQU	0x0161

; Temporary register used by envelope code
ETEMP1			EQU	0x0162

; Max			EQU	0x016F

;==================================================
; Memory - bank 3


; Note + velocity stacks
NOTE_STACK		EQU	0x0190
;				0x0191
;				0x0192
;				0x0193
;				0x0194
;				0x0195
;				0x0196
;				0x0197
;				0x0198
;				0x0199
;				0x019A
;				0x019B
;				0x019C
;				0x019D
;				0x019E
NOTE_STACK_BOTTOM	EQU	0x019F

; Portamento stuff
PORT1_STEPS_L		EQU	0x01A0
PORT1_STEPS_H		EQU	0x01A1
PORT2_STEPS_L		EQU	0x01A2
PORT2_STEPS_H		EQU	0x01A3
PORT3_STEPS_L		EQU	0x01A4
PORT3_STEPS_H		EQU	0x01A5

PORT1_STEP_AMT_LL	EQU	0x01A6
PORT1_STEP_AMT_L	EQU	0x01A7
PORT2_STEP_AMT_LL	EQU	0x01A8
PORT2_STEP_AMT_L	EQU	0x01A9
PORT3_STEP_AMT_LL	EQU	0x01AA
PORT3_STEP_AMT_L	EQU	0x01AB

PORT1_REM_AMT_LL	EQU	0x01AC
PORT1_REM_AMT_L		EQU	0x01AD
PORT2_REM_AMT_LL	EQU	0x01AE
PORT2_REM_AMT_L		EQU	0x01AF
PORT3_REM_AMT_LL	EQU	0x01B1
PORT3_REM_AMT_L		EQU	0x01B2

PORT_FLAGS		EQU	0x01B3
  P1_NEG		EQU	0x0
  P2_NEG		EQU	0x1
  P3_NEG		EQU	0x2

; Envelope stuff
ENV1_AMT_LL		EQU	0x01B4
ENV1_AMT_L		EQU	0x01B5
ENV1_AMT_H		EQU	0x01B6
ENV2_AMT_LL		EQU	0x01B7
ENV2_AMT_L		EQU	0x01B8
ENV2_AMT_H		EQU	0x01B9

ENV1_STEPS_L		EQU	0x01BA
ENV1_STEPS_H		EQU	0x01BB
ENV2_STEPS_L		EQU	0x01BC
ENV2_STEPS_H		EQU	0x01BD

ENV1_STEP_AMT_LL	EQU	0x01BE
ENV1_STEP_AMT_L		EQU	0x01BF
ENV1_STEP_AMT_H		EQU	0x01C0
ENV2_STEP_AMT_LL	EQU	0x01C1
ENV2_STEP_AMT_L		EQU	0x01C2
ENV2_STEP_AMT_H		EQU	0x01C3

ENV1_REM_AMT_LL		EQU	0x01C4
ENV1_REM_AMT_L		EQU	0x01C5
ENV2_REM_AMT_LL		EQU	0x01C6
ENV2_REM_AMT_L		EQU	0x01C7

ENV_FLAGS		EQU	0x01C8
  ENV1_NEG		EQU	0x0
  ENV2_NEG		EQU	0x1
  ENV1_PHASEA		EQU	0x2	; 0=Attack, 1=Decay/Sustain/Release
  ENV1_PHASEB		EQU	0x3	; 0=Decay/Sustain, 1=Release
  ENV2_PHASEA		EQU	0x4	; 0=Attack, 1=Decay/Sustain/Release
  ENV2_PHASEB		EQU	0x5	; 0=Decay/Sustain, 1=Release

; Max			EQU	0x01DF

; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0
; 0x00E0-0x00EF in bank 1
; 0x01E0-0x01EF in bank 3



;==================================================
; EEPROM

; EEPROM contains above bank 2 values at low byte of these addresses


EEPROM_FLAGS			EQU	0x00
  EEPROM_VALID			EQU	0x0	; Should be zero to indicate EEPROM contents is valid

EEPROM_ConfigMenuValuesBase	EQU	0x10 	; Up to 0x43

EEPROM_AssignMenuValuesBase	EQU	0x44	; Up to 0x61

EEPROM_Top			EQU	0x61	; Highest used EEPROM location
