;*********************************************************************
; FAT CONTROLLER ~ MIDI to CV Converter                              *
;*********************************************************************
; (c) 2002 SM-Sound / Seb Francis                                    *
;*********************************************************************
;
; Display Menu Code
;
;*********************************************************************


; --------------------------------
; Menu Functions
; Located in bank 0
;
; All subroutines must be called
; with RAM bank 0 selected
; --------------------------------


; Constants

MENU_CONFIG_LEN		EQU	D'40'
MENU_ASSIGN_LEN		EQU	D'3'


; --------------------------------


PageUp
	btfsc	MENU_FLAGS,WHICH_MENU	; Check which menu is currently active
	goto	PageUp_AssignMenu
	
	incf	MENU_CONFIG_POS		; Page up
	movfw	MENU_CONFIG_POS		; Test if wrapped
	sublw	MENU_CONFIG_LEN
	snz
	clrf	MENU_CONFIG_POS
	
	goto	UpdateDisplay		; Update display
	
PageUp_AssignMenu
	incf	MENU_ASSIGN_POS		; Page up
	movfw	MENU_ASSIGN_POS		; Test if wrapped
	sublw	MENU_ASSIGN_LEN
	snz
	clrf	MENU_ASSIGN_POS
	
	goto	UpdateDisplay		; Update display


; --------------------------------


PageDown
	btfsc	MENU_FLAGS,WHICH_MENU	; Check which menu is currently active
	goto	PageDown_AssignMenu
	
	decf	MENU_CONFIG_POS		; Page down
	btfss	MENU_CONFIG_POS,7	; Test if wrapped
	goto	UpdateDisplay
	movlw	MENU_CONFIG_LEN - 1
	movwf	MENU_CONFIG_POS
	
	goto	UpdateDisplay		; Update display
	
PageDown_AssignMenu
	decf	MENU_ASSIGN_POS		; Page down
	btfss	MENU_ASSIGN_POS,7	; Test if wrapped
	goto	UpdateDisplay
	movlw	MENU_ASSIGN_LEN - 1
	movwf	MENU_ASSIGN_POS
	
	goto	UpdateDisplay		; Update display


; --------------------------------


ResetValue
	bsf	STATUS,IRP		; Point INDF to banks 2-3
	
	btfsc	MENU_FLAGS,WHICH_MENU	; Check which menu is currently active
	goto	ResetValue_AssignMenu
	
	GVFT	MaxValues,MENU_CONFIG_POS	; Find if 16bit
	
	movwf	TEMP1			; If 0 (16bit value)
	
	GVFT	ValueAddresses,MENU_CONFIG_POS	; Get value address
	movwf	FSR
	
	GVFT	DefaultValues,MENU_CONFIG_POS	; Get default value
	movwf	INDF
	
	
	movfw	TEMP1			; Need to set high byte?
	jnz	UpdateDisplay_ValueOnly	; Update display and return
	
	
	movfw	INDF			; Get default value
	
	incf	FSR			; Point INDF to high value
	
	xorlw	0xFF			; Test if default = 0xFF
	movlw	0			; Clear MSB
	movwf	INDF
	jnz	UpdateDisplay_ValueOnly	; Update display and return
	movlw	0xFF			; If default = 0xFF, set MSB to 0xFF
	movwf	INDF
	goto	UpdateDisplay_ValueOnly	; Update display and return
	

ResetValue_AssignMenu
	movfw	MENU_ASSIGN_POS
	
	jz	ResetValue_AssignMenu_CV	; CV Out
	
	; All except CV page
	movwf	TEMP1
	bcf	STATUS,C
	rlf	TEMP1	; x2
	rlf	TEMP1	; x4
	rlf	TEMP1	; x8
	rlf	TEMP1	; x16
	subwf	TEMP1,w	; x15
	
	addwf	MENU_CURRENT_CV,w
	addlw	(low AssignMenuValuesBase) - D'16'
	movwf	FSR
	
	clrf	INDF
	
	goto	UpdateDisplay_ValueOnly
	
ResetValue_AssignMenu_CV
	movlw	1
	movwf	MENU_CURRENT_CV
	
	goto	UpdateDisplay_ValueOnly
	
; --------------------------------


IncrementValue
	bsf	STATUS,IRP		; Point INDF to banks 2-3
	
	btfsc	MENU_FLAGS,WHICH_MENU	; Check which menu is currently active
	goto	IncrementValue_AssignMenu
	
	GVFT	MaxValues,MENU_CONFIG_POS	; Get max value
	
	addlw	0			; Test if 0 (16bit value)
	jz	IncrementValue_16bit
	
	movwf	TEMP1			; Save max value
	
	GVFT	ValueAddresses,MENU_CONFIG_POS	; Get value address
	movwf	FSR
					
	movfw	INDF			; Test if at max
	subwf	TEMP1,w
	rz
	
	incf	INDF			; If not at max, increment value
	goto	UpdateDisplay_ValueOnly	; and update display
	
IncrementValue_16bit
	GVFT	ValueAddresses,MENU_CONFIG_POS	; Get low value address
	movwf	FSR
	
	btfsc	MENU_FLAGS,BIG_STEP		; Big step mode?
	goto	IncrementValue_16bit_BigStep
					
	incfsz	INDF			; Increment low value, check for carry
	goto	UpdateDisplay_ValueOnly
	
IncrementValue_16bit_1
	incf	FSR			; Point INDF to high value
					
	incfsz	INDF			; Increment high value, check for carry
	goto	UpdateDisplay_ValueOnly
	
	movlw	0xFF			; Wrap around not allowed - set back to 0xFFFF
	movwf	INDF
	decf	FSR
	movwf	INDF
	goto	UpdateDisplay_ValueOnly
	
IncrementValue_16bit_BigStep
	movlw	D'100'			; Add 100
	addwf	INDF
	btfss	STATUS,C		; No carry?
	goto	UpdateDisplay_ValueOnly
	goto	IncrementValue_16bit_1

IncrementValue_AssignMenu
	movfw	MENU_ASSIGN_POS
	
	jz	IncrementValue_AssignMenu_CV	; CV Out
	
	decf	MENU_ASSIGN_POS,w
	jz	IncrementValue_AssignMenu_Source	; Source
	
	; Amount
	movlw	(low vAMOUNT_CV1) - 1
	addwf	MENU_CURRENT_CV,w
	movwf	FSR
	
	incf	INDF
	movlw	D'126'
	subwf	INDF,w
	snz
	decf	INDF
	
	goto	UpdateDisplay_ValueOnly
	
IncrementValue_AssignMenu_CV
	incf	MENU_CURRENT_CV
	movlw	D'16'
	subwf	MENU_CURRENT_CV,w
	snz
	decf	MENU_CURRENT_CV
	
	goto	UpdateDisplay_ValueOnly
	
IncrementValue_AssignMenu_Source
	movlw	(low vSOURCE_CV1) - 1
	addwf	MENU_CURRENT_CV,w
	movwf	FSR
	
	incf	INDF
	movlw	D'131'
	subwf	INDF,w
	snz
	decf	INDF
	
	goto	UpdateDisplay_ValueOnly
	
; --------------------------------


DecrementValue
	bsf	STATUS,IRP		; Point INDF to banks 2-3
	
	btfsc	MENU_FLAGS,WHICH_MENU	; Check which menu is currently active
	goto	DecrementValue_AssignMenu
	
	GVFT	MinValues,MENU_CONFIG_POS	; Get min value
	movwf	TEMP1				; Save min value
	
	GVFT	MaxValues,MENU_CONFIG_POS	; Get max value
	
	addlw	0			; Test if 0 (16bit value)
	jz	DecrementValue_16bit
	
	GVFT	ValueAddresses,MENU_CONFIG_POS	; Get value address
	movwf	FSR
					
	movfw	INDF			; Test if at min
	subwf	TEMP1,w
	rz
	
	decf	INDF			; If not at min, decrement value
	goto	UpdateDisplay_ValueOnly	; Update display and return
	
	
DecrementValue_16bit
	GVFT	ValueAddresses,MENU_CONFIG_POS	; Get low value address
	movwf	FSR
	
	btfsc	MENU_FLAGS,BIG_STEP		; Big step mode?
	goto	DecrementValue_16bit_BigStep
	
	decf	INDF			; Decrement LSB
	incf	INDF,w			; Check for carry
	jz	DecrementValue_16bit_Carry
	
	movfw	TEMP1			; Test if < min
	subwf	INDF,w
	btfsc	STATUS,C		; Skip if < min
	goto	UpdateDisplay_ValueOnly	; Update display and return

	incf	FSR			; Point INDF to MSB
	movfw	INDF			; Test if MSB zero
	jz	DecrementValue_16bit_Wrap
	
	goto	UpdateDisplay_ValueOnly	; Update display and return
	
DecrementValue_16bit_Carry
	incf	FSR			; Point INDF to MSB
	movfw	INDF			; Test if MSB zero
	jz	DecrementValue_16bit_Wrap
	
	decf	INDF			; Decrement MSB
	goto	UpdateDisplay_ValueOnly	; Update display and return
	
	
	
DecrementValue_16bit_BigStep
	movlw	D'100'			; Subtract 100 from LSB
	subwf	INDF,f
	btfss	STATUS,C		; Skip if no borrow
	goto	DecrementValue_16bit_Borrow

	movfw	TEMP1			; Test if < min
	subwf	INDF,w
	btfsc	STATUS,C		; Skip if < min
	goto	UpdateDisplay_ValueOnly	; Update display and return
	
	incf	FSR			; Point INDF to MSB
	movfw	INDF			; Test if MSB zero
	jz	DecrementValue_16bit_Wrap
	
	goto	UpdateDisplay_ValueOnly	; Update display and return
	
DecrementValue_16bit_Borrow
	incf	FSR			; Point INDF to MSB
	movfw	INDF			; Test if MSB zero
	jz	DecrementValue_16bit_Wrap

	decf	INDF			; Decrement MSB
	goto	UpdateDisplay_ValueOnly	; Update display and return
	
	
	
DecrementValue_16bit_Wrap
	decf	FSR			; Point INDF to LSB
	movfw	TEMP1			; Set LSB to min
	movwf	INDF
	goto	UpdateDisplay_ValueOnly	; Update display and return
	
	
; --------------------------------

	
DecrementValue_AssignMenu
	movfw	MENU_ASSIGN_POS
	
	jz	DecrementValue_AssignMenu_CV	; CV Out
	
	decf	MENU_ASSIGN_POS,w
	jz	DecrementValue_AssignMenu_Source	; Source
	
	; Amount
	movlw	(low vAMOUNT_CV1) - 1
	addwf	MENU_CURRENT_CV,w
	movwf	FSR
	
	movlw	1
	subwf	INDF,f
	movlw	D'130'
	subwf	INDF,w
	snz
	incf	INDF
	
	goto	UpdateDisplay_ValueOnly
	
DecrementValue_AssignMenu_CV
	decf	MENU_CURRENT_CV
	snz
	incf	MENU_CURRENT_CV
	
	goto	UpdateDisplay_ValueOnly
	
DecrementValue_AssignMenu_Source
	movlw	(low vSOURCE_CV1) - 1
	addwf	MENU_CURRENT_CV,w
	movwf	FSR
	
	decf	INDF
	incf	INDF,w
	snz
	incf	INDF
	
	goto	UpdateDisplay_ValueOnly
	
; --------------------------------


SwitchMenu
	btfsc	MENU_FLAGS,WHICH_MENU	; Check which menu is currently active
	goto	SwitchMenu_AssignMenu
	
	bsf	MENU_FLAGS,WHICH_MENU	; Select assign menu
	
	goto	UpdateDisplay		; Update display
	
SwitchMenu_AssignMenu
	bcf	MENU_FLAGS,WHICH_MENU	; Select confg menu
	
	goto	UpdateDisplay		; Update display

	
; --------------------------------


SaveConfigModeOn
	; Display confirmation prompt
	movlw	sSAVE_CONFIG_T / 8		; Offset of top row string
	call	LCD_WriteTopRowString		; Write top row
	movlw	sSAVE_CONFIG_B / 8		; Offset of bottom row string
	call	LCD_WriteBottomRowString	; Write bottom row
	
	; Switch to 'save config' mode - suspends all other key operations (INCR to confirm, DECR to cancel)
	bsf	MENU_FLAGS,SAVE_CONFIG
	
	return
	
SaveConfigModeOff
	; Switch off 'save config' mode
	bcf	MENU_FLAGS,SAVE_CONFIG
	
	goto	UpdateDisplay		; Update display
	
SaveConfigModeGo
	; Indicate to main loop that save confirmed
	bsf	MENU_FLAGS,SAVE_CONFIG_GO
	
	goto	UpdateDisplay		; Update display

	
; --------------------------------


UpdateDisplay
	btfsc	MENU_FLAGS,WHICH_MENU	; Check which menu is currently active
	goto	UpdateDisplay_AssignMenu
	
	movlw	ConfigMenuStrings / 8	; Start offset of menu strings
	addwf	MENU_CONFIG_POS,w	; Add offset for current page
	call	LCD_WriteTopRowString	; Write top row
	
UpdateDisplay_VO
	movfw	MENU_CONFIG_POS		; Offset = 2x menu pos
	addwf	MENU_CONFIG_POS,w
	movwf	TEMP1			; Save offset
	movlw	high UpdateDisplay_JumpTable
	movwf	PCLATH
	movlw	low UpdateDisplay_JumpTable
	addwf	TEMP1,w
	snc
	incf	PCLATH
	bsf	STATUS,IRP		; Set up IRP ready for bottom row code
	movwf	PCL			; Jump to code to write bottom row
	
UpdateDisplay_AssignMenu
	movlw	AssignMenuStrings / 8	; Start offset of menu strings
	addwf	MENU_ASSIGN_POS,w	; Add offset for current page
	call	LCD_WriteTopRowString

UpdateDisplay_AssignMenu_VO
	movfw	MENU_ASSIGN_POS
	
	bsf	STATUS,IRP		; Set up IRP ready for bottom row code
	
	jz	UpdateDisplay_AssignMenu_CV	; CV Out
	
	decf	MENU_ASSIGN_POS,w
	jz	UpdateDisplay_AssignMenu_Source	; Source
	
	; Amount
	movlw	(low vAMOUNT_CV1) - 1
	addwf	MENU_CURRENT_CV,w
	goto	UD_vt_75V_75V
	
UpdateDisplay_AssignMenu_CV
	bcf	STATUS,IRP
	movlw	low MENU_CURRENT_CV 
	goto	UD_vt_CV1_CV15
	
UpdateDisplay_AssignMenu_Source
	movlw	(low vSOURCE_CV1) - 1
	addwf	MENU_CURRENT_CV,w
	goto	UD_vt_CV_Source
	
; --------------------------------
	

UpdateDisplay_ValueOnly
	btfsc	MENU_FLAGS,WHICH_MENU	; Check which menu is currently active
	goto	UpdateDisplay_AssignMenu_VO

	goto	UpdateDisplay_VO

	
; --------------------------------


; Table of code routines for writing bottom row of each menu page

UpdateDisplay_JumpTable

	movlw	low vBASE_CHA
	goto	UD_vt_1_16
	
	movlw	low vCTRL_CHA
	goto	UD_vt_Off_1_16
	
	movlw	low vMODE
	goto	UD_vt_nCha_Poly
	
	movlw	low vL_RETRIG
	goto	UD_vt_Off_On
	
	movlw	low vN_O_GATE
	goto	UD_vt_Off_10ms_50s
	
	movlw	low vVELOCITY
	goto	UD_vt_Off_003V_750V
	
	movlw	low vPORTA
	goto	UD_vt_Off_Legato_On
	
	movlw	low vPORTA_T1
	goto	UD_vt_40ms_50s
	
	movlw	low vPORTA_T2
	goto	UD_vt_Off_40ms_50s
	
	movlw	low vPORTA_T3
	goto	UD_vt_Off_40ms_50s
	
	movlw	low vP_BEND_1
	goto	UD_vt_0_127
	
	movlw	low vP_BEND_2
	goto	UD_vt_Off_0_127
	
	movlw	low vP_BEND_3
	goto	UD_vt_Off_0_127
	
	movlw	low vOCTAVE_1
	goto	UD_vt_7_7
	
	movlw	low vOCTAVE_2
	goto	UD_vt_7_Off_7
	
	movlw	low vOCTAVE_3
	goto	UD_vt_7_Off_7
	
	movlw	low vLFO1_TIM
	goto	UD_vt_10ms_655s
	
	movlw	low vLFO1_WAV
	goto	UD_vt_WaveShape
	
	movlw	low vLFO1_SYN
	goto	UD_vt_Sync
	
	movlw	low vLFO1_AMT
	goto	UD_vt_Off_003V_750V
	
	movlw	low vLFO1_DST
	goto	UD_vt_CV1_CV15
	
	movlw	low vLFO2_TIM
	goto	UD_vt_10ms_655s
	
	movlw	low vLFO2_WAV
	goto	UD_vt_WaveShape
	
	movlw	low vLFO2_SYN
	goto	UD_vt_Sync
	
	movlw	low vLFO2_AMT
	goto	UD_vt_Off_003V_750V
	
	movlw	low vLFO2_DST
	goto	UD_vt_CV1_CV15
	
	movlw	low vENV1_TRG
	goto	UD_vt_Off_Gate1_Gate3
	
	movlw	low vENV1_ATT
	goto	UD_vt_0ms_50s
	
	movlw	low vENV1_ATL
	goto	UD_vt_75V_75V
	
	movlw	low vENV1_DEC
	goto	UD_vt_0ms_50s
	
	movlw	low vENV1_SUS
	goto	UD_vt_75V_75V
	
	movlw	low vENV1_REL
	goto	UD_vt_0ms_50s
	
	movlw	low vENV1_DST
	goto	UD_vt_CV1_CV15
	
	movlw	low vENV2_TRG
	goto	UD_vt_Off_Gate1_Gate3
	
	movlw	low vENV2_ATT
	goto	UD_vt_0ms_50s
	
	movlw	low vENV2_ATL
	goto	UD_vt_75V_75V
	
	movlw	low vENV2_DEC
	goto	UD_vt_0ms_50s
	
	movlw	low vENV2_SUS
	goto	UD_vt_75V_75V
	
	movlw	low vENV2_REL
	goto	UD_vt_0ms_50s
	
	movlw	low vENV2_DST
	goto	UD_vt_CV1_CV15
	
	
; --------------------------------


; Code routines to write each type of value to bottom row

; Display: 1-16
; Store: 0-15
UD_vt_1_16
	movwf	FSR			; Point to value, IRP is already set
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movlw	1			; Write value + 1
	addwf	INDF,w
	call	LCD_WriteInt
	
	movlw	7			; Blank rest of line and return
	goto	LCD_WriteSpaces


; Display: 1 Cha / 2 Cha / 3 Cha / Poly
; Store: 0-3
UD_vt_nCha_Poly
	movwf	FSR			; Point to value, IRP is already set
	
	movlw	s1_Cha / 8
	addwf	INDF,w
	goto	LCD_WriteBottomRowString


; Display: Off / On
; Store: 0-1
UD_vt_Off_On
	movwf	FSR			; Point to value, IRP is already set
	
	movlw	sOff2 / 8
	addwf	INDF,w
	goto	LCD_WriteBottomRowString


; Display: Off / 0.01s-50.00s
; Store: 0 / 0x0001-0x1388
UD_vt_Off_10ms_50s
	movwf	FSR			; Point to value, IRP is already set
	
	; Test if MSB=0
	incf	FSR
	movfw	INDF
	jz	UD_vt_Off_10ms_50s_Small
	
	decf	FSR			; Point back to LSB
	
	; Test if > 0x1388
	movfw	INDF
	sublw	low 0x1388
	incf	FSR
	movfw	INDF
	sc
	incfsz	INDF,w
	sublw	high 0x1388		; If w bit 7 set, value is too big
	
	movwf	TEMP1
	btfsc	TEMP1,7
	goto	UD_vt_Off_10ms_50s_TooBig
	
	decf	FSR			; Point back to LSB
	
UD_vt_Off_10ms_50s_1
	call	LCD_StartBottomRow	; Set cursor position
	
	call	LCD_WriteLongIntDiv100	; Write value

	movlw	"s"			; Write "s"
	call	LCD_WriteDR
	
	movlw	3			; Blank rest of line and return
	goto	LCD_WriteSpaces
	
UD_vt_Off_10ms_50s_TooBig
	movlw	high 0x1388		; Set value to max
	movwf	INDF
	decf	FSR
	movlw	low 0x1388
	movwf	INDF
	
	goto	UD_vt_Off_10ms_50s_1

UD_vt_Off_10ms_50s_Small
	decf	FSR			; Point back to LSB
	
	; If LSB=0, write "Off" and return
	movfw	INDF
	movlw	sOff / 8
	jz	LCD_WriteBottomRowString
	
	; Jump to write value
	goto	UD_vt_Off_10ms_50s_1


; Display: Off / 0.03V-7.50V
; Store: 0 / 1-250
UD_vt_Off_003V_750V
	movwf	FSR			; Point to value, IRP is already set
	
	movfw	INDF			; If 0, write "Off"
	movlw	sOff / 8
	jz	LCD_WriteBottomRowString
	
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movfw	INDF			; value * 3 / 100 => n.nn
	movwf	AARGB0
	movlw	3
	movwf	BARGB0
	bsf	PCLATH,4	; Select program bank 2 (0x1000 - 0x17FF)
	call	FXM0808U
	bcf	PCLATH,4	; Select program bank 0 (0x0000 - 0x07FF)
	
	movlw	low AARGB1
	movwf	FSR
	bcf	STATUS,IRP		; Clear IRP to access banks 0-1 via INDF
	call	LCD_WriteLongIntDiv100	; Write value as n.nn
	
	movlw	"V"			; Write "V"
	call	LCD_WriteDR
	
	movlw	3			; Blank rest of line and return
	goto	LCD_WriteSpaces


; Display: Off / 1-16
; Store: 0 / 1-16
UD_vt_Off_1_16
	movwf	FSR			; Point to value, IRP is already set
	
	movfw	INDF			; If 0, write "Off"
	movlw	sOff / 8
	jz	LCD_WriteBottomRowString
	
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movfw	INDF			; Write value
	call	LCD_WriteInt
	
	movlw	7			; Blank rest of line and return
	goto	LCD_WriteSpaces
	

; Display: Off / Legato 1 / Legato 2 / Legato 3 / On
; Store: 0-4
UD_vt_Off_Legato_On
	movwf	FSR			; Point to value, IRP is already set
	
	movlw	sOff / 8
	addwf	INDF,w
	goto	LCD_WriteBottomRowString


; Display: 0.04s-50.00s
; Store: 0x0004-0x1388
UD_vt_40ms_50s
	movwf	FSR			; Point to value, IRP is already set
	
	; Test if > 0x1388
	movfw	INDF
	sublw	low 0x1388
	incf	FSR
	movfw	INDF
	sc
	incfsz	INDF,w
	sublw	high 0x1388		; If w bit 7 set, value is too big
	
	movwf	TEMP1
	btfsc	TEMP1,7
	goto	UD_vt_40ms_50s_TooBig
	
	decf	FSR			; Point back to LSB
	
UD_vt_40ms_50s_1
	call	LCD_StartBottomRow	; Set cursor position
	
	call	LCD_WriteLongIntDiv100	; Write value

	movlw	"s"			; Write "s"
	call	LCD_WriteDR
	
	movlw	3			; Blank rest of line and return
	goto	LCD_WriteSpaces
	
UD_vt_40ms_50s_TooBig
	movlw	high 0x1388		; Set value to max
	movwf	INDF
	decf	FSR
	movlw	low 0x1388
	movwf	INDF
	
	goto	UD_vt_40ms_50s_1


; Display: Off / 0.04s-50.00s
; Store: 0 / 0x0004-0x1388
UD_vt_Off_40ms_50s
	movwf	FSR			; Point to value, IRP is already set
	
	; Test if MSB=0
	incf	FSR
	movfw	INDF
	jz	UD_vt_Off_40ms_50s_Small
	
	decf	FSR			; Point back to LSB
	
	; Test if > 0x1388
	movfw	INDF
	sublw	low 0x1388
	incf	FSR
	movfw	INDF
	sc
	incfsz	INDF,w
	sublw	high 0x1388		; If w bit 7 set, value is too big
	
	movwf	TEMP1
	btfsc	TEMP1,7
	goto	UD_vt_Off_40ms_50s_TooBig
	
	decf	FSR			; Point back to LSB
	
UD_vt_Off_40ms_50s_1
	call	LCD_StartBottomRow	; Set cursor position
	
	call	LCD_WriteLongIntDiv100	; Write value

	movlw	"s"			; Write "s"
	call	LCD_WriteDR
	
	movlw	3			; Blank rest of line and return
	goto	LCD_WriteSpaces
	
UD_vt_Off_40ms_50s_TooBig
	movlw	high 0x1388		; Set value to max
	movwf	INDF
	decf	FSR
	movlw	low 0x1388
	movwf	INDF
	
	goto	UD_vt_Off_40ms_50s_1

UD_vt_Off_40ms_50s_Small
	decf	FSR			; Point back to LSB
	
	; If LSB=0, write "Off" and return
	movfw	INDF
	movlw	sOff / 8
	jz	LCD_WriteBottomRowString
	
	; If LSB=1, set LSB to 4
	decf	INDF,w
	movlw	4
	snz
	movwf	INDF
	
	; If LSB!=3, jump to write value
	movlw	3
	subwf	INDF,w
	jnz	UD_vt_Off_40ms_50s_1
	
	; Otherwise set LSB to 0, write "Off" and return
	clrf	INDF
	movlw	sOff / 8
	goto	LCD_WriteBottomRowString
	

; Display: 0-127
; Store: 1-128
UD_vt_0_127
	movwf	FSR			; Point to value, IRP is already set
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movlw	1			; Write value - 1
	subwf	INDF,w
	call	LCD_WriteInt
	
	movlw	7			; Blank rest of line and return
	goto	LCD_WriteSpaces


; Display: Off / 0-127
; Store: 0-128
UD_vt_Off_0_127
	movwf	FSR			; Point to value, IRP is already set
	
	movfw	INDF			; If 0, write "Off"
	movlw	sOff / 8
	jz	LCD_WriteBottomRowString
	
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movlw	1			; Write value - 1
	subwf	INDF,w
	call	LCD_WriteInt
	
	movlw	7			; Blank rest of line and return
	goto	LCD_WriteSpaces


; Display: -7-0-7
; Store: 249-0-7
UD_vt_7_7
	movwf	FSR			; Point to value, IRP is already set
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movfw	INDF
	btfss	INDF,7			; Test if +ve
	goto	UD_vt_7_7_Positive
	
	movlw	"-"			; Write "-"
	call	LCD_WriteDR
	
	movfw	INDF
	sublw	0			; Flip sign of -ve value
	
UD_vt_7_7_Positive
	call	LCD_WriteInt
	
	movlw	7			; Blank rest of line and return
	goto	LCD_WriteSpaces


; Display: -7-Off-7
; Store: 249-0-7
UD_vt_7_Off_7
	movwf	FSR			; Point to value, IRP is already set
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movfw	INDF
	jz	UD_vt_7_Off_7_Off	; Test if 0
	
	btfss	INDF,7			; Test if +ve
	goto	UD_vt_7_Off_7_Positive
	
	movlw	"-"			; Write "-"
	call	LCD_WriteDR
	
	movfw	INDF
	sublw	0			; Flip sign of -ve value
	
UD_vt_7_Off_7_Positive
	call	LCD_WriteInt
	
	movlw	7			; Blank rest of line and return
	goto	LCD_WriteSpaces

UD_vt_7_Off_7_Off
	movlw	sOff / 8
	goto	LCD_WriteBottomRowString


; Display: 0.00s-655.35s
; Store: 0-65535
UD_vt_10ms_655s
	movwf	FSR			; Point to value, IRP is already set
	
	call	LCD_StartBottomRow	; Set cursor position
	
	call	LCD_WriteLongIntDiv100	; Write value

	movlw	"s"			; Write "s"
	call	LCD_WriteDR
	
	movlw	3			; Blank rest of line and return
	goto	LCD_WriteSpaces


; Display: 0.00s-50.00s
; Store: 0x0000-0x1388
UD_vt_0ms_50s
	movwf	FSR			; Point to LSB, IRP is already set
	
	; Test if > 0x1388
	movfw	INDF
	sublw	low 0x1388
	incf	FSR
	movfw	INDF
	sc
	incfsz	INDF,w
	sublw	high 0x1388		; If w bit 7 set, value is too big
	
	movwf	TEMP1
	btfsc	TEMP1,7
	goto	UD_vt_0ms_50s_TooBig
	
	decf	FSR			; Point back to LSB
	
UD_vt_0ms_50s_1
	call	LCD_StartBottomRow	; Set cursor position
	
	call	LCD_WriteLongIntDiv100	; Write value

	movlw	"s"			; Write "s"
	call	LCD_WriteDR
	
	movlw	3			; Blank rest of line and return
	goto	LCD_WriteSpaces
	
UD_vt_0ms_50s_TooBig
	movlw	high 0x1388		; Set value to max
	movwf	INDF
	decf	FSR
	movlw	low 0x1388
	movwf	INDF
	
	goto	UD_vt_0ms_50s_1


; Display: Sine / Triangle / Saw Down / Saw Up / Square / Random
; Store: 0-5
UD_vt_WaveShape
	movwf	FSR			; Point to value, IRP is already set
	
	movlw	sSine / 8
	addwf	INDF,w
	goto	LCD_WriteBottomRowString
	

; Display: Off / 1/64 / 1/32 / 1/16 / 1/8 / 3/16 / 1/4 / 5/16 / 3/8 / 7/16 / 1/2 / 9/16 / 5/8 / 11/16 / 3/4 / 13/16 / 7/8 / 15/16 / 1 / 4/3 / 3/2 / 5/3 / 2 / 7/3 / 5/2 / 8/3 / 3 / 4 / 8 / 16 / 32 / 64
; Store: 0-31
UD_vt_Sync
	movwf	FSR			; Point to value, IRP is already set
	
	movfw	INDF			; If 0, write "Off"
	movlw	sOff / 8
	jz	LCD_WriteBottomRowString
	
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movlw	"S"			; Write "Sync:"
	call	LCD_WriteDR
	movlw	"y"
	call	LCD_WriteDR
	movlw	"n"
	call	LCD_WriteDR
	movlw	"c"
	call	LCD_WriteDR
	movlw	":"
	call	LCD_WriteDR
	
	movfw	INDF			; Write value
	call	LCD_WriteInt
	
	movlw	2			; Blank rest of line and return
	goto	LCD_WriteSpaces


; Display: CV1-CV15
; Store: 1-15
UD_vt_CV1_CV15
	movwf	FSR			; Point to value, IRP is already set
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movlw	"C"			; Write "C"
	call	LCD_WriteDR
	movlw	"V"			; Write "V"
	call	LCD_WriteDR
	
	movfw	INDF			; Write value
	call	LCD_WriteInt
	
	movlw	5			; Blank rest of line and return
	goto	LCD_WriteSpaces


; Display: Off / Gate 1-3
; Store: 0-3
UD_vt_Off_Gate1_Gate3
	movwf	FSR			; Point to value, IRP is already set
	
	movfw	INDF			; Write "Off" ?
	jz	UD_vt_Off_Gate1_Gate3_Off
	
	movlw	(sGate_1 / 8) - 1
	addwf	INDF,w
	goto	LCD_WriteBottomRowString
	
UD_vt_Off_Gate1_Gate3_Off
	movlw	sOff / 8
	goto	LCD_WriteBottomRowString
	

; Display: -7.5V-0V-7.5V
; Store: 131-0-125
UD_vt_75V_75V
	movwf	FSR			; Point to value, IRP is already set
	
	call	LCD_StartBottomRow	; Set cursor position
	
	movfw	INDF
	btfss	INDF,7			; Test >= 0
	goto	UD_vt_75V_75V_Positive
	
	movlw	"-"			; Write "-"
	call	LCD_WriteDR
	
	movfw	INDF			; Restore w
	sublw	0			; Flip sign of -ve value
	
UD_vt_75V_75V_Positive
	
	movwf	AARGB0			; w * 6 / 100 => n.nn
	movlw	6
	movwf	BARGB0
	bsf	PCLATH,4	; Select program bank 2 (0x1000 - 0x17FF)
	call	FXM0808U
	bcf	PCLATH,4	; Select program bank 0 (0x0000 - 0x07FF)
	
	movlw	low AARGB1
	movwf	FSR
	bcf	STATUS,IRP		; Clear IRP to access banks 0-1 via INDF
	call	LCD_WriteLongIntDiv100	; Write value as n.nn
	
	movlw	"V"			; Write "V"
	call	LCD_WriteDR
	
	movlw	3			; Blank rest of line and return
	goto	LCD_WriteSpaces
	

; Display: Off / P Bend / Pressure / CC Learn / CC 0-127
; Store: 0 / 1 / 2 / 3 / 4-131
UD_vt_CV_Source
	movwf	FSR			; Point to value, IRP is already set
	
	movlw	4			; If 0-3, write string
	subwf	INDF,w
	movwf	TEMP1
	btfss	TEMP1,7
	goto	UD_vt_CV_Source_CC
	
	movlw	sOff3 / 8
	addwf	INDF,w
	goto	LCD_WriteBottomRowString
	
UD_vt_CV_Source_CC
	call	LCD_StartBottomRow	; Set cursor position
	
	movlw	"C"			; Write "CC "
	call	LCD_WriteDR
	movlw	"C"
	call	LCD_WriteDR
	movlw	" "
	call	LCD_WriteDR
	
	movfw	TEMP1			; Write value
	call	LCD_WriteInt
	
	movlw	4			; Blank rest of line and return
	goto	LCD_WriteSpaces
	
; --------------------------------


; Subroutine to get value (used by macro GVFT)
; PCLATH must be already set
GetValue
	movwf	PCL

; Table of addresses of values of each menu page

ValueAddresses
	retlw	low vBASE_CHA
	retlw	low vCTRL_CHA
	retlw	low vMODE
	retlw	low vL_RETRIG
	retlw	low vN_O_GATE
	retlw	low vVELOCITY
	retlw	low vPORTA
	retlw	low vPORTA_T1
	retlw	low vPORTA_T2
	retlw	low vPORTA_T3
	retlw	low vP_BEND_1
	retlw	low vP_BEND_2
	retlw	low vP_BEND_3
	retlw	low vOCTAVE_1
	retlw	low vOCTAVE_2
	retlw	low vOCTAVE_3
	retlw	low vLFO1_TIM
	retlw	low vLFO1_WAV
	retlw	low vLFO1_SYN
	retlw	low vLFO1_AMT
	retlw	low vLFO1_DST
	retlw	low vLFO2_TIM
	retlw	low vLFO2_WAV
	retlw	low vLFO2_SYN
	retlw	low vLFO2_AMT
	retlw	low vLFO2_DST
	retlw	low vENV1_TRG
	retlw	low vENV1_ATT
	retlw	low vENV1_ATL
	retlw	low vENV1_DEC
	retlw	low vENV1_SUS
	retlw	low vENV1_REL
	retlw	low vENV1_DST
	retlw	low vENV2_TRG
	retlw	low vENV2_ATT
	retlw	low vENV2_ATL
	retlw	low vENV2_DEC
	retlw	low vENV2_SUS
	retlw	low vENV2_REL
	retlw	low vENV2_DST
	
	
; Table of min values of each menu page

MinValues
	retlw	D'0'	; vBASE_CHA
	retlw	D'0'	; vCTRL_CHA
	retlw	D'0'	; vMODE
	retlw	D'0'	; vL_RETRIG
	retlw	D'0'	; vN_O_GATE
	retlw	D'0'	; vVELOCITY
	retlw	D'0'	; vPORTA
	retlw	D'4'	; vPORTA_T1
	retlw	D'0'	; vPORTA_T2
	retlw	D'0'	; vPORTA_T3
	retlw	D'1'	; vP_BEND_1
	retlw	D'0'	; vP_BEND_2
	retlw	D'0'	; vP_BEND_3
	retlw	D'249'	; vOCTAVE_1
	retlw	D'249'	; vOCTAVE_2
	retlw	D'249'	; vOCTAVE_3
	retlw	D'1'	; vLFO1_TIM
	retlw	D'0'	; vLFO1_WAV
	retlw	D'0'	; vLFO1_SYN
	retlw	D'0'	; vLFO1_AMT
	retlw	D'1'	; vLFO1_DST
	retlw	D'1'	; vLFO2_TIM
	retlw	D'0'	; vLFO2_WAV
	retlw	D'0'	; vLFO2_SYN
	retlw	D'0'	; vLFO2_AMT
	retlw	D'1'	; vLFO2_DST
	retlw	D'0'	; vENV1_TRG
	retlw	D'0'	; vENV1_ATT
	retlw	D'131'	; vENV1_ATL
	retlw	D'0'	; vENV1_DEC
	retlw	D'131'	; vENV1_SUS
	retlw	D'0'	; vENV1_REL
	retlw	D'1'	; vENV1_DST
	retlw	D'0'	; vENV2_TRG
	retlw	D'0'	; vENV2_ATT
	retlw	D'131'	; vENV2_ATL
	retlw	D'0'	; vENV2_DEC
	retlw	D'131'	; vENV2_SUS
	retlw	D'0'	; vENV2_REL
	retlw	D'1'	; vENV2_DST
	
	
; Table of max values of each menu page
; 0 means 16bit max value

MaxValues
	retlw	D'15'	; vBASE_CHA
	retlw	D'16'	; vCTRL_CHA
	retlw	D'3'	; vMODE
	retlw	D'1'	; vL_RETRIG
	retlw	D'0'	; vN_O_GATE
	retlw	D'250'	; vVELOCITY
	retlw	D'4'	; vPORTA
	retlw	D'0'	; vPORTA_T1
	retlw	D'0'	; vPORTA_T2
	retlw	D'0'	; vPORTA_T3
	retlw	D'128'	; vP_BEND_1
	retlw	D'128'	; vP_BEND_2
	retlw	D'128'	; vP_BEND_3
	retlw	D'7'	; vOCTAVE_1
	retlw	D'7'	; vOCTAVE_2
	retlw	D'7'	; vOCTAVE_3
	retlw	D'0'	; vLFO1_TIM
	retlw	D'5'	; vLFO1_WAV
	retlw	D'31'	; vLFO1_SYN
	retlw	D'250'	; vLFO1_AMT
	retlw	D'15'	; vLFO1_DST
	retlw	D'0'	; vLFO2_TIM
	retlw	D'5'	; vLFO2_WAV
	retlw	D'31'	; vLFO2_SYN
	retlw	D'250'	; vLFO2_AMT
	retlw	D'15'	; vLFO2_DST
	retlw	D'3'	; vENV1_TRG
	retlw	D'0'	; vENV1_ATT
	retlw	D'125'	; vENV1_ATL
	retlw	D'0'	; vENV1_DEC
	retlw	D'125'	; vENV1_SUS
	retlw	D'0'	; vENV1_REL
	retlw	D'15'	; vENV1_DST
	retlw	D'3'	; vENV2_TRG
	retlw	D'0'	; vENV2_ATT
	retlw	D'125'	; vENV2_ATL
	retlw	D'0'	; vENV2_DEC
	retlw	D'125'	; vENV2_SUS
	retlw	D'0'	; vENV2_REL
	retlw	D'15'	; vENV2_DST
	
	
; Table of default values of each menu page
; 255 means 65535 for 16bit values

DefaultValues
	retlw	D'0'	; vBASE_CHA
	retlw	D'0'	; vCTRL_CHA
	retlw	D'0'	; vMODE
	retlw	D'0'	; vL_RETRIG
	retlw	D'0'	; vN_O_GATE
	retlw	D'0'	; vVELOCITY
	retlw	D'0'	; vPORTA
	retlw	D'25'	; vPORTA_T1
	retlw	D'0'	; vPORTA_T2
	retlw	D'0'	; vPORTA_T3
	retlw	D'6'	; vP_BEND_1
	retlw	D'0'	; vP_BEND_2
	retlw	D'0'	; vP_BEND_3
	retlw	D'0'	; vOCTAVE_1
	retlw	D'0'	; vOCTAVE_2
	retlw	D'0'	; vOCTAVE_3
	retlw	D'200'	; vLFO1_TIM
	retlw	D'0'	; vLFO1_WAV
	retlw	D'0'	; vLFO1_SYN
	retlw	D'0'	; vLFO1_AMT
	retlw	D'1'	; vLFO1_DST
	retlw	D'200'	; vLFO2_TIM
	retlw	D'0'	; vLFO2_WAV
	retlw	D'0'	; vLFO2_SYN
	retlw	D'0'	; vLFO2_AMT
	retlw	D'1'	; vLFO2_DST
	retlw	D'0'	; vENV1_TRG
	retlw	D'0'	; vENV1_ATT
	retlw	D'125'	; vENV1_ATL
	retlw	D'10'	; vENV1_DEC
	retlw	D'125'	; vENV1_SUS
	retlw	D'25'	; vENV1_REL
	retlw	D'1'	; vENV1_DST
	retlw	D'0'	; vENV2_TRG
	retlw	D'0'	; vENV2_ATT
	retlw	D'125'	; vENV2_ATL
	retlw	D'10'	; vENV2_DEC
	retlw	D'125'	; vENV2_SUS
	retlw	D'25'	; vENV2_REL
	retlw	D'1'	; vENV2_DST


; --------------------------------
